@@ -1,8 +1,6 @@
 #ifndef foosaphfoo
 #define foosaphfoo
 
-/* $Id: sap.h 1465 2007-05-29 17:24:48Z lennart $ */
-
 /***
   This file is part of PulseAudio.
 
@@ -40,9 +38,9 @@ typedef struct pa_sap_context {
 pa_sap_context* pa_sap_context_init_send(pa_sap_context *c, int fd, char *sdp_data);
 void pa_sap_context_destroy(pa_sap_context *c);
 
-int pa_sap_send(pa_sap_context *c, int goodbye);
+int pa_sap_send(pa_sap_context *c, pa_bool_t goodbye);
 
 pa_sap_context* pa_sap_context_init_recv(pa_sap_context *c, int fd);
-int pa_sap_recv(pa_sap_context *c, int *goodbye);
+int pa_sap_recv(pa_sap_context *c, pa_bool_t *goodbye);
 
 #endif
