@@ -1,5 +1,3 @@
-/* $Id: channelmap.c 2054 2007-11-13 23:41:17Z lennart $ */
-
 /***
   This file is part of PulseAudio.
 
@@ -31,6 +29,7 @@
 #include <string.h>
 
 #include <pulse/xmalloc.h>
+#include <pulse/i18n.h>
 #include <pulsecore/core-util.h>
 #include <pulsecore/macro.h>
 
@@ -100,66 +99,66 @@ const char *const table[PA_CHANNEL_POSITION_MAX] = {
 };
 
 const char *const pretty_table[PA_CHANNEL_POSITION_MAX] = {
-    [PA_CHANNEL_POSITION_MONO] = "Mono",
+    [PA_CHANNEL_POSITION_MONO] = N_("Mono"),
 
-    [PA_CHANNEL_POSITION_FRONT_CENTER] = "Front Center",
-    [PA_CHANNEL_POSITION_FRONT_LEFT] = "Front Left",
-    [PA_CHANNEL_POSITION_FRONT_RIGHT] = "Front Right",
+    [PA_CHANNEL_POSITION_FRONT_CENTER] = N_("Front Center"),
+    [PA_CHANNEL_POSITION_FRONT_LEFT] = N_("Front Left"),
+    [PA_CHANNEL_POSITION_FRONT_RIGHT] = N_("Front Right"),
 
-    [PA_CHANNEL_POSITION_REAR_CENTER] = "Rear Center",
-    [PA_CHANNEL_POSITION_REAR_LEFT] = "Rear Left",
-    [PA_CHANNEL_POSITION_REAR_RIGHT] = "Rear Right",
+    [PA_CHANNEL_POSITION_REAR_CENTER] = N_("Rear Center"),
+    [PA_CHANNEL_POSITION_REAR_LEFT] = N_("Rear Left"),
+    [PA_CHANNEL_POSITION_REAR_RIGHT] = N_("Rear Right"),
 
-    [PA_CHANNEL_POSITION_LFE] = "Low Frequency Emmiter",
+    [PA_CHANNEL_POSITION_LFE] = N_("Low Frequency Emmiter"),
 
-    [PA_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER] = "Front Left-of-center",
-    [PA_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER] = "Front Right-of-center",
+    [PA_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER] = N_("Front Left-of-center"),
+    [PA_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER] = N_("Front Right-of-center"),
 
-    [PA_CHANNEL_POSITION_SIDE_LEFT] = "Side Left",
-    [PA_CHANNEL_POSITION_SIDE_RIGHT] = "Side Right",
+    [PA_CHANNEL_POSITION_SIDE_LEFT] = N_("Side Left"),
+    [PA_CHANNEL_POSITION_SIDE_RIGHT] = N_("Side Right"),
 
-    [PA_CHANNEL_POSITION_AUX0] = "Auxiliary 0",
-    [PA_CHANNEL_POSITION_AUX1] = "Auxiliary 1",
-    [PA_CHANNEL_POSITION_AUX2] = "Auxiliary 2",
-    [PA_CHANNEL_POSITION_AUX3] = "Auxiliary 3",
-    [PA_CHANNEL_POSITION_AUX4] = "Auxiliary 4",
-    [PA_CHANNEL_POSITION_AUX5] = "Auxiliary 5",
-    [PA_CHANNEL_POSITION_AUX6] = "Auxiliary 6",
-    [PA_CHANNEL_POSITION_AUX7] = "Auxiliary 7",
-    [PA_CHANNEL_POSITION_AUX8] = "Auxiliary 8",
-    [PA_CHANNEL_POSITION_AUX9] = "Auxiliary 9",
-    [PA_CHANNEL_POSITION_AUX10] = "Auxiliary 10",
-    [PA_CHANNEL_POSITION_AUX11] = "Auxiliary 11",
-    [PA_CHANNEL_POSITION_AUX12] = "Auxiliary 12",
-    [PA_CHANNEL_POSITION_AUX13] = "Auxiliary 13",
-    [PA_CHANNEL_POSITION_AUX14] = "Auxiliary 14",
-    [PA_CHANNEL_POSITION_AUX15] = "Auxiliary 15",
-    [PA_CHANNEL_POSITION_AUX16] = "Auxiliary 16",
-    [PA_CHANNEL_POSITION_AUX17] = "Auxiliary 17",
-    [PA_CHANNEL_POSITION_AUX18] = "Auxiliary 18",
-    [PA_CHANNEL_POSITION_AUX19] = "Auxiliary 19",
-    [PA_CHANNEL_POSITION_AUX20] = "Auxiliary 20",
-    [PA_CHANNEL_POSITION_AUX21] = "Auxiliary 21",
-    [PA_CHANNEL_POSITION_AUX22] = "Auxiliary 22",
-    [PA_CHANNEL_POSITION_AUX23] = "Auxiliary 23",
-    [PA_CHANNEL_POSITION_AUX24] = "Auxiliary 24",
-    [PA_CHANNEL_POSITION_AUX25] = "Auxiliary 25",
-    [PA_CHANNEL_POSITION_AUX26] = "Auxiliary 26",
-    [PA_CHANNEL_POSITION_AUX27] = "Auxiliary 27",
-    [PA_CHANNEL_POSITION_AUX28] = "Auxiliary 28",
-    [PA_CHANNEL_POSITION_AUX29] = "Auxiliary 29",
-    [PA_CHANNEL_POSITION_AUX30] = "Auxiliary 30",
-    [PA_CHANNEL_POSITION_AUX31] = "Auxiliary 31",
+    [PA_CHANNEL_POSITION_AUX0] = N_("Auxiliary 0"),
+    [PA_CHANNEL_POSITION_AUX1] = N_("Auxiliary 1"),
+    [PA_CHANNEL_POSITION_AUX2] = N_("Auxiliary 2"),
+    [PA_CHANNEL_POSITION_AUX3] = N_("Auxiliary 3"),
+    [PA_CHANNEL_POSITION_AUX4] = N_("Auxiliary 4"),
+    [PA_CHANNEL_POSITION_AUX5] = N_("Auxiliary 5"),
+    [PA_CHANNEL_POSITION_AUX6] = N_("Auxiliary 6"),
+    [PA_CHANNEL_POSITION_AUX7] = N_("Auxiliary 7"),
+    [PA_CHANNEL_POSITION_AUX8] = N_("Auxiliary 8"),
+    [PA_CHANNEL_POSITION_AUX9] = N_("Auxiliary 9"),
+    [PA_CHANNEL_POSITION_AUX10] = N_("Auxiliary 10"),
+    [PA_CHANNEL_POSITION_AUX11] = N_("Auxiliary 11"),
+    [PA_CHANNEL_POSITION_AUX12] = N_("Auxiliary 12"),
+    [PA_CHANNEL_POSITION_AUX13] = N_("Auxiliary 13"),
+    [PA_CHANNEL_POSITION_AUX14] = N_("Auxiliary 14"),
+    [PA_CHANNEL_POSITION_AUX15] = N_("Auxiliary 15"),
+    [PA_CHANNEL_POSITION_AUX16] = N_("Auxiliary 16"),
+    [PA_CHANNEL_POSITION_AUX17] = N_("Auxiliary 17"),
+    [PA_CHANNEL_POSITION_AUX18] = N_("Auxiliary 18"),
+    [PA_CHANNEL_POSITION_AUX19] = N_("Auxiliary 19"),
+    [PA_CHANNEL_POSITION_AUX20] = N_("Auxiliary 20"),
+    [PA_CHANNEL_POSITION_AUX21] = N_("Auxiliary 21"),
+    [PA_CHANNEL_POSITION_AUX22] = N_("Auxiliary 22"),
+    [PA_CHANNEL_POSITION_AUX23] = N_("Auxiliary 23"),
+    [PA_CHANNEL_POSITION_AUX24] = N_("Auxiliary 24"),
+    [PA_CHANNEL_POSITION_AUX25] = N_("Auxiliary 25"),
+    [PA_CHANNEL_POSITION_AUX26] = N_("Auxiliary 26"),
+    [PA_CHANNEL_POSITION_AUX27] = N_("Auxiliary 27"),
+    [PA_CHANNEL_POSITION_AUX28] = N_("Auxiliary 28"),
+    [PA_CHANNEL_POSITION_AUX29] = N_("Auxiliary 29"),
+    [PA_CHANNEL_POSITION_AUX30] = N_("Auxiliary 30"),
+    [PA_CHANNEL_POSITION_AUX31] = N_("Auxiliary 31"),
 
-    [PA_CHANNEL_POSITION_TOP_CENTER] = "Top Center",
+    [PA_CHANNEL_POSITION_TOP_CENTER] = N_("Top Center"),
 
-    [PA_CHANNEL_POSITION_TOP_FRONT_CENTER] = "Top Front Center",
-    [PA_CHANNEL_POSITION_TOP_FRONT_LEFT] = "Top Front Left",
-    [PA_CHANNEL_POSITION_TOP_FRONT_RIGHT] = "Top Front Right",
+    [PA_CHANNEL_POSITION_TOP_FRONT_CENTER] = N_("Top Front Center"),
+    [PA_CHANNEL_POSITION_TOP_FRONT_LEFT] = N_("Top Front Left"),
+    [PA_CHANNEL_POSITION_TOP_FRONT_RIGHT] = N_("Top Front Right"),
 
-    [PA_CHANNEL_POSITION_TOP_REAR_CENTER] = "Top Rear Center",
-    [PA_CHANNEL_POSITION_TOP_REAR_LEFT] = "Top Rear left",
-    [PA_CHANNEL_POSITION_TOP_REAR_RIGHT] = "Top Rear Right"
+    [PA_CHANNEL_POSITION_TOP_REAR_CENTER] = N_("Top Rear Center"),
+    [PA_CHANNEL_POSITION_TOP_REAR_LEFT] = N_("Top Rear Left"),
+    [PA_CHANNEL_POSITION_TOP_REAR_RIGHT] = N_("Top Rear Right")
 };
 
 pa_channel_map* pa_channel_map_init(pa_channel_map *m) {
@@ -199,10 +198,11 @@ pa_channel_map* pa_channel_map_init_auto(pa_channel_map *m, unsigned channels, p
     pa_assert(m);
     pa_assert(channels > 0);
     pa_assert(channels <= PA_CHANNELS_MAX);
+    pa_assert(def < PA_CHANNEL_MAP_DEF_MAX);
 
     pa_channel_map_init(m);
 
-    m->channels = channels;
+    m->channels = (uint8_t) channels;
 
     switch (def) {
         case PA_CHANNEL_MAP_AIFF:
@@ -288,9 +288,6 @@ pa_channel_map* pa_channel_map_init_auto(pa_channel_map *m, unsigned channels, p
         case PA_CHANNEL_MAP_AUX: {
             unsigned i;
 
-            if (channels >= PA_CHANNELS_MAX)
-                return NULL;
-
             for (i = 0; i < channels; i++)
                 m->map[i] = PA_CHANNEL_POSITION_AUX0 + i;
 
@@ -392,10 +389,39 @@ pa_channel_map* pa_channel_map_init_auto(pa_channel_map *m, unsigned channels, p
 
 
         default:
-            return NULL;
+            pa_assert_not_reached();
     }
 }
 
+pa_channel_map* pa_channel_map_init_extend(pa_channel_map *m, unsigned channels, pa_channel_map_def_t def) {
+    unsigned c;
+
+    pa_assert(m);
+    pa_assert(channels > 0);
+    pa_assert(channels <= PA_CHANNELS_MAX);
+    pa_assert(def < PA_CHANNEL_MAP_DEF_MAX);
+
+    pa_channel_map_init(m);
+
+    for (c = channels; c > 0; c--) {
+
+        if (pa_channel_map_init_auto(m, c, def)) {
+            unsigned i = 0;
+
+            for (; c < channels; c++) {
+
+                m->map[c] = PA_CHANNEL_POSITION_AUX0 + i;
+                i++;
+            }
+
+            m->channels = (uint8_t) channels;
+
+            return m;
+        }
+    }
+
+    return NULL;
+}
 
 const char* pa_channel_position_to_string(pa_channel_position_t pos) {
 
@@ -406,10 +432,13 @@ const char* pa_channel_position_to_string(pa_channel_position_t pos) {
 }
 
 const char* pa_channel_position_to_pretty_string(pa_channel_position_t pos) {
+
+    pa_init_i18n();
+
     if (pos < 0 || pos >= PA_CHANNEL_POSITION_MAX)
         return NULL;
 
-    return pretty_table[pos];
+    return _(pretty_table[pos]);
 }
 
 int pa_channel_map_equal(const pa_channel_map *a, const pa_channel_map *b) {
@@ -430,13 +459,20 @@ int pa_channel_map_equal(const pa_channel_map *a, const pa_channel_map *b) {
 
 char* pa_channel_map_snprint(char *s, size_t l, const pa_channel_map *map) {
     unsigned channel;
-    int first = 1;
+    pa_bool_t first = TRUE;
     char *e;
 
     pa_assert(s);
     pa_assert(l > 0);
     pa_assert(map);
 
+    pa_init_i18n();
+
+    if (!pa_channel_map_valid(map)) {
+        pa_snprintf(s, l, _("(invalid)"));
+        return s;
+    }
+
     *(e = s) = 0;
 
     for (channel = 0; channel < map->channels && l > 1; channel++) {
@@ -445,7 +481,7 @@ char* pa_channel_map_snprint(char *s, size_t l, const pa_channel_map *map) {
                       pa_channel_position_to_string(map->map[channel]));
 
         e = strchr(e, 0);
-        first = 0;
+        first = FALSE;
     }
 
     return s;
@@ -459,7 +495,7 @@ pa_channel_map *pa_channel_map_parse(pa_channel_map *rmap, const char *s) {
     pa_assert(rmap);
     pa_assert(s);
 
-    memset(&map, 0, sizeof(map));
+    pa_channel_map_init(&map);
 
     if (strcmp(s, "stereo") == 0) {
         map.channels = 2;
@@ -522,12 +558,22 @@ int pa_channel_map_valid(const pa_channel_map *map) {
     if (map->channels <= 0 || map->channels > PA_CHANNELS_MAX)
         return 0;
 
-    for (c = 0; c < map->channels; c++) {
-
-        if (map->map[c] < 0 ||map->map[c] >= PA_CHANNEL_POSITION_MAX)
+    for (c = 0; c < map->channels; c++)
+        if (map->map[c] < 0 || map->map[c] >= PA_CHANNEL_POSITION_MAX)
             return 0;
 
-    }
-
     return 1;
 }
+
+int pa_channel_map_compatible(const pa_channel_map *map, const pa_sample_spec *ss) {
+    pa_assert(map);
+    pa_assert(ss);
+
+    if (!pa_channel_map_valid(map))
+        return 0;
+
+    if (!pa_sample_spec_valid(ss))
+        return 0;
+
+    return map->channels == ss->channels;
+}
