@@ -1,8 +1,6 @@
 #ifndef fooclihfoo
 #define fooclihfoo
 
-/* $Id: cli.h 1426 2007-02-13 15:35:19Z ossman $ */
-
 /***
   This file is part of PulseAudio.
 
@@ -30,11 +28,15 @@
 
 typedef struct pa_cli pa_cli;
 
+typedef void (*pa_cli_eof_cb_t)(pa_cli *c, void *userdata);
+
 /* Create a new command line session on the specified io channel owned by the specified module */
 pa_cli* pa_cli_new(pa_core *core, pa_iochannel *io, pa_module *m);
 void pa_cli_free(pa_cli *cli);
 
 /* Set a callback function that is called whenever the command line session is terminated */
-void pa_cli_set_eof_callback(pa_cli *cli, void (*cb)(pa_cli*c, void *userdata), void *userdata);
+void pa_cli_set_eof_callback(pa_cli *cli, pa_cli_eof_cb_t cb, void *userdata);
+
+pa_module *pa_cli_get_module(pa_cli *c);
 
 #endif
