@@ -1,8 +1,6 @@
 #ifndef foomodinfohfoo
 #define foomodinfohfoo
 
-/* $Id: modinfo.h 1426 2007-02-13 15:35:19Z ossman $ */
-
 /***
   This file is part of PulseAudio.
 
@@ -25,16 +23,18 @@
 ***/
 
 /* Some functions for reading module meta data from PulseAudio modules */
+#include <pulsecore/macro.h>
 
 typedef struct pa_modinfo {
     char *author;
     char *description;
     char *usage;
     char *version;
+    pa_bool_t load_once;
 } pa_modinfo;
 
 /* Read meta data from an libtool handle */
-pa_modinfo *pa_modinfo_get_by_handle(lt_dlhandle dl);
+pa_modinfo *pa_modinfo_get_by_handle(lt_dlhandle dl, const char *module_name);
 
 /* Read meta data from a module file */
 pa_modinfo *pa_modinfo_get_by_name(const char *name);
