@@ -1,5 +1,3 @@
-/* $Id: parseaddr.c 1426 2007-02-13 15:35:19Z ossman $ */
-
 /***
   This file is part of PulseAudio.
 
@@ -26,13 +24,13 @@
 #endif
 
 #include <string.h>
-#include <assert.h>
 #include <stdlib.h>
 
 #include <pulse/xmalloc.h>
-
 #include <pulse/util.h>
+
 #include <pulsecore/core-util.h>
+#include <pulsecore/macro.h>
 
 #include "parseaddr.h"
 
@@ -45,32 +43,46 @@
  *  Return a newly allocated string of the hostname and fill in *ret_port if specified  */
 
 static char *parse_host(const char *s, uint16_t *ret_port) {
-    assert(s && ret_port);
+    pa_assert(s);
+    pa_assert(ret_port);
+
     if (*s == '[') {
         char *e;
         if (!(e = strchr(s+1, ']')))
             return NULL;
 
-        if (e[1] == ':')
-            *ret_port = atoi(e+2);
-        else if (e[1] != 0)
+        if (e[1] == ':') {
+            uint32_t p;
+
+            if (pa_atou(e+2, &p) < 0)
+                return NULL;
+
+            *ret_port = (uint16_t) p;
+        } else if (e[1] != 0)
             return NULL;
 
-        return pa_xstrndup(s+1, e-s-1);
+        return pa_xstrndup(s+1, (size_t) (e-s-1));
     } else {
         char *e;
+        uint32_t p;
 
         if (!(e = strrchr(s, ':')))
             return pa_xstrdup(s);
 
-        *ret_port = atoi(e+1);
-        return pa_xstrndup(s, e-s);
+        if (pa_atou(e+1, &p) < 0)
+            return NULL;
+
+        *ret_port = (uint16_t) p;
+        return pa_xstrndup(s, (size_t) (e-s));
     }
 }
 
 int pa_parse_address(const char *name, pa_parsed_address *ret_p) {
     const char *p;
-    assert(name && ret_p);
+
+    pa_assert(name);
+    pa_assert(ret_p);
+
     memset(ret_p, 0, sizeof(pa_parsed_address));
     ret_p->type = PA_PARSED_ADDRESS_TCP_AUTO;
 
@@ -98,9 +110,12 @@ int pa_parse_address(const char *name, pa_parsed_address *ret_p) {
     else if (pa_startswith(p, "unix:")) {
         ret_p->type = PA_PARSED_ADDRESS_UNIX;
         p += sizeof("unix:")-1;
-    } else if (pa_startswith(p, "tcp:") || pa_startswith(p, "tcp4:")) {
+    } else if (pa_startswith(p, "tcp:")) {
         ret_p->type = PA_PARSED_ADDRESS_TCP4;
         p += sizeof("tcp:")-1;
+    } else if (pa_startswith(p, "tcp4:")) {
+        ret_p->type = PA_PARSED_ADDRESS_TCP4;
+        p += sizeof("tcp4:")-1;
     } else if (pa_startswith(p, "tcp6:")) {
         ret_p->type = PA_PARSED_ADDRESS_TCP6;
         p += sizeof("tcp6:")-1;
@@ -112,6 +127,5 @@ int pa_parse_address(const char *name, pa_parsed_address *ret_p) {
         if (!(ret_p->path_or_host = parse_host(p, &ret_p->port)))
             return -1;
 
-
     return 0;
 }
