@@ -1,8 +1,6 @@
 #ifndef foopidhfoo
 #define foopidhfoo
 
-/* $Id: pid.h 2067 2007-11-21 01:30:40Z lennart $ */
-
 /***
   This file is part of PulseAudio.
 
@@ -24,9 +22,9 @@
   USA.
 ***/
 
-int pa_pid_file_create(void);
+int pa_pid_file_create(const char *procname);
 int pa_pid_file_remove(void);
-int pa_pid_file_check_running(pid_t *pid, const char *binary_name);
-int pa_pid_file_kill(int sig, pid_t *pid, const char *binary_name);
+int pa_pid_file_check_running(pid_t *pid, const char *procname);
+int pa_pid_file_kill(int sig, pid_t *pid, const char *procname);
 
 #endif
