@@ -1,5 +1,3 @@
-/* $Id: poll.c 1426 2007-02-13 15:35:19Z ossman $ */
-
 
 /***
   This file is part of PulseAudio.
@@ -45,7 +43,7 @@
 
 #include "winsock.h"
 
-#ifndef HAVE_SYS_POLL_H
+#ifndef HAVE_POLL_H
 
 #include <pulsecore/core-util.h>
 
